@@ -15,6 +15,8 @@ sub new {
     bless $self, $class;
 
     $self->_dir($top->_current_components_copy());
+    $self->_stat_ret($top->_top_stat_copy());
+
     $self->idx($index);
 
     $self->_last_dir_scanned(undef);
@@ -50,11 +52,14 @@ use base 'File::Find::Object::Base';
 
 use File::Find::Object::Result;
 
+use Fcntl ':mode';
+
 __PACKAGE__->mk_accessors(qw(
     _dir_stack
     item_obj
     _targets
     _target_index
+    _top_stat
 ));
 
 sub _get_options_ids
@@ -112,12 +117,13 @@ __PACKAGE__->_top_it([qw(
 
 __PACKAGE__->_make_copy_methods([qw(
     _current_components
+    _top_stat
     )]
 );
 
 use Carp;
 
-our $VERSION = '0.1.4';
+our $VERSION = '0.1.5';
 
 sub new {
     my ($class, $options, @targets) = @_;
@@ -169,6 +175,14 @@ sub _is_top
     return ! @{$self->_dir_stack()};
 }
 
+sub _curr_mode {
+    return shift->_top_stat->[2];
+}
+
+sub _curr_not_a_dir {
+    return !S_ISDIR( shift->_curr_mode() );
+}
+
 sub _current_path
 {
     my $self = shift;
@@ -181,11 +195,12 @@ sub _calc_current_item_obj {
 
     my $components = $self->_current_components_copy();
     my $base = shift(@$components);
-    
-    my @basename = ();
+    my $stat = $self->_top_stat_copy();
+
     my $path = $self->_current_path();
-    my $is_dir = -d $path;
-    if (! $is_dir)
+
+    my @basename = ();
+    if ($self->_curr_not_a_dir())
     {
         @basename = (basename => pop(@$components));
     }
@@ -194,9 +209,9 @@ sub _calc_current_item_obj {
         {
             @basename,
             path => $path,
-            is_dir => $is_dir,
             dir_components => $components,
             base => $base,
+            stat_ret => $stat,
         }
     );
 }
@@ -358,7 +373,15 @@ sub _calc_actions
     {
         @actions = reverse(@actions);
     }
-    return @actions;
+    return ("_mystat", @actions);
+}
+
+sub _mystat {
+    my $self = shift;
+
+    $self->_top_stat([stat($self->_current_path())]);
+
+    return "SKIP";
 }
 
 sub _get_real_action
@@ -466,14 +489,20 @@ sub _check_subdir
 
     # If current is not a directory always return 0, because we may
     # be asked to traverse single-files.
-    my @st = stat($self->_current_path());
-    if (!-d _)
+
+    if ($self->_is_top()) {
+        # Assign to _stat_ret as well, so the _stat_ret field of the top
+        # item will be set.    
+        $self->_stat_ret($self->_top_stat_copy());
+    }
+
+    if ($self->_curr_not_a_dir())
     {
         return 0;
     }
     else
     {
-        return $self->_check_subdir_helper(\@st);
+        return $self->_check_subdir_helper();
     }
 }
 
@@ -483,12 +512,11 @@ sub _top__check_subdir_helper {
 
 sub _find_ancestor_with_same_inode {
     my $self = shift;
-    my $st = shift;
 
     my $ptr = $self->_current_father;
 
     while($ptr) {
-        if ($ptr->_is_same_inode($st)) {
+        if ($ptr->_is_same_inode($self->_top_stat())) {
             return $ptr;
         }
     }
@@ -518,19 +546,20 @@ sub _warn_about_loop
 
 sub _non_top__check_subdir_helper {
     my $self = shift;
-    my $st = shift;
 
-    if (-l $self->_current_path() && !$self->followlink())
+    if (S_ISLNK($self->_curr_mode()) && !$self->followlink())
     {
         return 0;
     }
 
-    if ($st->[0] != $self->_current_father->_dev() && $self->nocrossfs())
+    if (   $self->_top_stat->[0] != $self->_current_father->_dev()
+        && $self->nocrossfs()
+    )
     {
         return 0;
     }
 
-    if (my $ptr = $self->_find_ancestor_with_same_inode($st)) {
+    if (my $ptr = $self->_find_ancestor_with_same_inode()) {
         $self->_warn_about_loop($ptr);
         return 0;
     }
